@@ -6,8 +6,23 @@
 #include <QProcess>
 #include <QtDBus>
 #include <QDesktopServices>
+
+//#include <QLibrary>
+
+//#include <QVersitReader>
+//#include <QVersitDocument>
+//#include <QBuffer>
+//#include <QContact>
+//#include <QContactManager>
+
 #include "textwindow.h"
 
+#include <hildon-mime.h>
+
+//#define QT_NO_KEYWORDS
+//#include <libosso-abook/osso-abook-contact.h>
+//#include <libosso-abook/osso-abook-util.h>
+
 QrSink::QrSink(const PluginInterface *interface) : SinkPlugin(interface) {
     ready = false;
 }
@@ -23,6 +38,7 @@ void QrSink::clickAction(QWidget *parentWindow) {
      //interface->call("open_new_window",url);
 
     if(type==QrSink::vCard){
+        addVCard(barcodeText); //openGeneric(barcodeText); //
 
     }else if(type==QrSink::BizCard){
 
@@ -30,7 +46,7 @@ void QrSink::clickAction(QWidget *parentWindow) {
         openInstallFile(barcodeText);
 
     }else if(type==QrSink::URL){
-        openGeneric(barcodeText);
+        openBrowser(barcodeText);
 
     }else if(type==QrSink::iMode){
     }else if(type==QrSink::Email){
@@ -40,7 +56,17 @@ void QrSink::clickAction(QWidget *parentWindow) {
         openGeneric(barcodeText);
 
     }else if(type==QrSink::SMS){
-        openGeneric(barcodeText);
+        if(!barcodeText.contains("SMS://",Qt::CaseInsensitive)){
+            // change the uri from SMS: / SMSTO: to SMS://
+            QString t(barcodeText);
+            if(barcodeText.contains("SMS:",Qt::CaseInsensitive)){
+                t.replace("SMS:", "SMS://", Qt::CaseInsensitive);
+            }else if(barcodeText.contains("SMSTO:",Qt::CaseInsensitive)){
+                t.replace("SMSTO:", "SMS://", Qt::CaseInsensitive);
+            }
+            openGeneric(t);
+        }else
+            openGeneric(barcodeText);
 
     }else if(type==QrSink::MMS){
         openGeneric(barcodeText);
@@ -74,94 +100,94 @@ void QrSink::barcodeAnalysed(QString barcodeType, QString barcodeData) {
         // it would be better to parse the data before we display our button, that way we could show the url, email, tel no, etc. on the button
         // something for the improved version... :)
 
-        if (barcodeData.indexOf("BEGIN:VCARD")==0){
+        if (barcodeData.indexOf("BEGIN:VCARD",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::vCard;
             buttonText.append("Open vCard");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("BIZCARD:")==0){
+        }else if(barcodeData.indexOf("BIZCARD:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::BizCard;
             buttonText.append("Open BizCard");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("[install]")==0){
+        }else if(barcodeData.indexOf("[install]",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::Install;
             buttonText.append("Install package");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("URL:")==0){
+        }else if(barcodeData.indexOf("URL:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::URL;
             buttonText.append("Open URL");
             barcodeText=barcodeData;
-        }else if(barcodeData.indexOf("URLTO:")==0){
+        }else if(barcodeData.indexOf("URLTO:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::URL;
             buttonText.append("Open URL");
             barcodeText=barcodeData;
-        }else if(barcodeData.contains("http://") || barcodeData.contains("https://")){
+        }else if(barcodeData.indexOf("http://",0,Qt::CaseInsensitive)==0 || barcodeData.indexOf("https://",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::URL;
             buttonText.append("Open URL");
             barcodeText=barcodeData; // need to find where the http:// occurs in the string and trim it!
 
-        }else if(barcodeData.indexOf("CNTS:")==0){
+        }else if(barcodeData.indexOf("CNTS:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::iMode;
             buttonText.append("Open iMode data");
             barcodeText=barcodeData;
-        }else if(barcodeData.indexOf("MECARD:")==0){
+        }else if(barcodeData.indexOf("MECARD:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::iMode;
             buttonText.append("Open iMode data");
             barcodeText=barcodeData;
-        }else if(barcodeData.indexOf("MATMSG:")==0){
+        }else if(barcodeData.indexOf("MATMSG:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::iMode;
             buttonText.append("Open iMode data");
             barcodeText=barcodeData;
-        }else if(barcodeData.indexOf("MEBKM:")==0){
+        }else if(barcodeData.indexOf("MEBKM:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::iMode;
             buttonText.append("Open iMode data");
             barcodeText=barcodeData;
-        }else if(barcodeData.indexOf("LAPL:")==0){
+        }else if(barcodeData.indexOf("LAPL:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::iMode;
             buttonText.append("Open iMode data");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("mailto:")==0){
+        }else if(barcodeData.indexOf("mailto:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::Email;
             buttonText.append("Send e-mail");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("tel:")==0){
+        }else if(barcodeData.indexOf("tel:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::PhoneNumber;
             buttonText.append("Use phone number");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("sms:")==0){
+        }else if(barcodeData.indexOf("sms:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::SMS;
             buttonText.append("Open SMS");
             barcodeText=barcodeData;
-        }else if(barcodeData.indexOf("smsto:")==0){
+        }else if(barcodeData.indexOf("smsto:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::SMS;
             buttonText.append("Open SMS");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("mms:")==0){
+        }else if(barcodeData.indexOf("mms:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::MMS;
             buttonText.append("Open MMS");
             barcodeText=barcodeData;
-        }else if(barcodeData.indexOf("mmsto:")==0){
+        }else if(barcodeData.indexOf("mmsto:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::MMS;
             buttonText.append("Open MMS");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("geo:")==0){
+        }else if(barcodeData.indexOf("geo:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::Geo;
             buttonText.append("Open geo location");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("youtube:")==0){
+        }else if(barcodeData.indexOf("youtube:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::YouTube;
             buttonText.append("Open YouTube link");
             barcodeText=barcodeData;
 
-        }else if(barcodeData.indexOf("ovi:")==0){
+        }else if(barcodeData.indexOf("ovi:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::Ovi;
             buttonText.append("Install from Ovi");
             barcodeText=barcodeData;
@@ -351,6 +377,40 @@ void QrSink::openInstallFile(QString data)
 
 
 void QrSink::openGeneric(QString uri)
+{
+
+    qDebug() << "In openGeneric()";
+
+    GError *error; // see if we can get away with this...
+    //HildonURIAction* action = hildon_uri_get_default_action_by_uri((const gchar*)uri.constData(), &error);
+    //hildon_uri_open((const gchar*)uri.constData(), action, &error);
+
+    qDebug() << "openGeneric(): Trying to handle: " << uri.toAscii();
+
+    hildon_uri_open((const gchar*)uri.constData(), NULL, NULL);
+
+    //hildon_uri_open ((const gchar*)uri.constData(), HILDON_URI_ACTION_NORMAL, &error);
+
+    //QDesktopServices::openUrl(uri);
+
+//#ifdef Q_WS_MAEMO_5 // Uses native file opening method
+//            //TODO: find better solution for this, maybe get fixed in Qt
+//            DBusConnection* conn;
+//            QFile file(uri);
+//            conn = dbus_bus_get(DBUS_BUS_SESSION, 0);
+//            hildon_mime_open_file(conn, QUrl::fromLocalFile(file.absoluteFilePath()).toEncoded().constData());
+//#else
+//            /*
+//            Not working with maemo5.
+//            Uses hildon_uri_open function from
+//            libhildonmime which should work,
+//            but all files opened in browser.
+//            */
+//            QDesktopServices::openUrl(QUrl::fromLocalFile(file.absoluteFilePath()));
+//#endif
+}
+
+void QrSink::openBrowser(QString uri)
 {
     //action = hildon_uri_get_default_action_by_uri(uri, &error);
     //hildon_uri_open(uri, action, &error); //hildon_uri_open (uri, HILDON_URI_ACTION_NORMAL, error);
@@ -365,6 +425,53 @@ void QrSink::openGeneric(QString uri)
 // this uses GLib stuff, I suppose we should try to convert it to use Qt completely.....
 void QrSink::addVCard(QString user_data)
 {
+    QDir d("/home/user/.mbarcode/temp/");
+    if(!d.exists("/home/user/.mbarcode/temp/"))
+        d.mkpath("/home/user/.mbarcode/temp/");
+
+    QTemporaryFile file("/home/user/.mbarcode/temp/qt_tempXXXXXX.vcf");
+    file.open();
+    file.write(user_data.toAscii());
+    file.close();
+
+    file.setAutoRemove(false); // we'll do this at the end
+
+    openGeneric(QString("file://" + file.fileName()));
+
+
+    //file.~QTemporaryFile();
+
+
+    /*QVersitReader reader;
+    QBuffer b(user_data.toAscii(), this);
+    b.open(QBuffer::ReadOnly);
+    reader.setDevice(&b);
+
+    reader.readAll();
+
+    QVersitDocument inputDocument = reader.result().takeFirst();
+    QVersitContactImporter importer;
+    QContact contact = importer.importContact(inputDocument);
+
+    QContactManager::saveContact(contact); */
+
+
+
+//    g_type_init();
+//    QLibrary *&lib = qgconf_libgconf;
+//    qDebug() << "loading gconf";;
+//    lib = new QLibrary(QLatin1String("gconf-2"));
+//    triedToLoadLibrary = true;
+//    if (lib->load() && lib->resolve("gconf_client_get_default")) {
+//        qDebug() << "load successfull!";
+//    } else {
+//        qDebug() << "Load failed";
+//        lib->unload();
+//        delete lib;
+//        lib = 0;
+//        return;
+//    }
+//
 //    char *vcard = (char*)user_data.constData();
 //    OssoABookContact *contact;
 //    GError *error = NULL;
@@ -382,6 +489,9 @@ void QrSink::addVCard(QString user_data)
 //            g_warning ("%s: %s", G_STRLOC, error->message);
 //            g_assert_not_reached ();
 //    }
+//
+//    lib->unload();
+//    delete lib;
 }
 
 
